"""Write out the rpc_info.py file."""
from collections import defaultdict
from os import environ
from typing import List

from proto_parsing_utils import ProtoParserException, RpcData, parse_all_protos_in_dir

tab = "    "
ansys_api_edb_repo_env_str = "ANSYS_API_EDB_REPO_PATH"


def _get_rpc_info_object_str(data: RpcData):
    io_flag_params = ", ".join(
        [f"{io_flag}={True}" for io_flag in sorted(data.io_flags, key=str.lower)]
    )
    return f"_RpcInfo({io_flag_params})"


def _rpc_info_to_str_entries(rpc_datas: List[RpcData]):
    rpc_info_str_entries = []
    for data in rpc_datas:
        rpc_info_str_entries.append(
            f"""{tab}\"{data.rpc_name}\": {_get_rpc_info_object_str(data)}"""
        )
    rpc_info_str_entries = ",\n".join(rpc_info_str_entries)
    return f"{{{rpc_info_str_entries}}}"


def _rpc_info_to_str(rpc_datas: List[RpcData]):
    service_to_rpc_map = defaultdict(list)
    for data in rpc_datas:
        service_to_rpc_map[data.full_service_name].append(data)
    rpc_info_str = []
    for service_name, data in service_to_rpc_map.items():
        service_info_str = f"""{tab}\"{service_name}\": {_rpc_info_to_str_entries(data)}"""
        rpc_info_str.append(service_info_str)
    return f",\n".join(rpc_info_str)


def _get_rpc_info_file_str(rpc_datas: List[RpcData]):
    return f"""\"\"\"Defines container which gives additional information for RPC methods.\"\"\"


class _RpcInfo:
    def __init__(
        self,
        read_no_cache=False,
        write_no_buffer=False,
        cache=False,
        buffer=False,
        returns_future=False,
        write_no_cache_invalidation=False,
    ):
        self._read_no_cache = read_no_cache
        self._write_no_buffer = write_no_buffer
        self._cache = cache
        self._buffer = buffer
        self._write_no_cache_invalidation = write_no_cache_invalidation
        self._returns_future = returns_future

    @property
    def is_read(self):
        return self._cache or self._read_no_cache

    @property
    def is_write(self):
        return self._buffer or self._write_no_buffer

    @property
    def can_cache(self):
        return self._cache

    @property
    def can_buffer(self):
        return self._buffer

    @property
    def returns_future(self):
        return self._returns_future

    @property
    def invalidates_cache(self):
        return self.is_write and not self._write_no_cache_invalidation


rpc_information = {{
{_rpc_info_to_str(rpc_datas)}
}}

"""


def _write_rpc_info_file(rpc_datas: List[RpcData], path: str):
    with open(path, "w") as f:
        f.write(_get_rpc_info_file_str(rpc_datas))


if __name__ == "__main__":
    try:
        rpc_data = []
        ansys_api_edb_repo = environ.get(ansys_api_edb_repo_env_str)
        if not ansys_api_edb_repo:
            raise ProtoParserException(
                f"The environment variable {ansys_api_edb_repo_env_str} is not set. "
                f"Please set it to the path of the ansys-api-edb repository on your system "
                f"so that the proto files can be successfully parsed."
            )
        parse_all_protos_in_dir(ansys_api_edb_repo + r"/ansys/api/edb/v1", rpc_data)
        _write_rpc_info_file(rpc_data, r"../../src/ansys/edb/core/inner/rpc_info.py")
    except ProtoParserException as e:
        print(e)



"""HFSS simulation settings."""

from enum import Enum

import ansys.api.edb.v1.hfss_pi_simulation_settings_pb2 as pb

from ansys.edb.core.inner import messages
from ansys.edb.core.session import (
    HFSSPIAdvancedSettingsServiceStub,
    HFSSPIGeneralSettingsServiceStub,
    HFSSPISolverSettingsServiceStub,
    StubAccessor,
    StubType,
)
from ansys.edb.core.simulation_setup.simulation_settings import (
    AdvancedMeshingSettingsServiceStub,
    AdvancedSettingsServiceStub,
    SimulationSettings,
    SimulationSettingsBase,
)


class HFSSPIModelType(Enum):
    """Provides an enum representing HFSS adaptive solution types."""

    PCB = pb.HFSSPI_PCB
    PACKAGE = pb.HFSSPI_PACKAGE
    RDL = pb.HFSSPI_RDL


class HFSSPISimulationSettings(SimulationSettings):
    """Represents HFSSPI simulation settings."""

    @property
    def general(self):
        """:class:`.HFSSPIGeneralSettings`: General settings for HFSSPI simulations."""
        return HFSSPIGeneralSettings(self)

    @property
    def advanced(self):
        """:class:`.HFSSPIAdvancedSettings`: Advanced settings for HFSSPI simulations."""
        return HFSSPIAdvancedSettings(self)

    @property
    def solver(self):
        """:class:`.HFSSPISolverSettings`: Solver settings for HFSSPI simulations."""
        return HFSSPISolverSettings(self)


class HFSSPIGeneralSettings(SimulationSettingsBase):
    """Represents general settings for HFSSPI simulations."""

    __stub: HFSSPIGeneralSettingsServiceStub = StubAccessor(StubType.hfss_pi_general_sim_settings)

    @property
    def model_type(self):
        """:class:`.HFSSPIModelType`: Model type."""
        return HFSSPIModelType(self.__stub.GetHFSSPIModelType(self.msg).hfss_pi_model_type)

    @model_type.setter
    def model_type(self, hfss_pi_model_type):
        self.__stub.SetHFSSPIModelType(
            pb.HFSSPIModelTypePropertyMessage(
                target=self.msg, hfss_pi_model_type=hfss_pi_model_type.value
            )
        )

    @property
    def use_auto_mesh_region(self):
        """:obj:`bool`: Flag indicating if auto mesh regions are used."""
        return self.__stub.GetUseAutoMeshRegion(self.msg).value

    @use_auto_mesh_region.setter
    def use_auto_mesh_region(self, use_auto_mesh_region):
        self.__stub.SetUseAutoMeshRegion(messages.bool_property_message(self, use_auto_mesh_region))

    @property
    def use_mesh_region(self):
        """:obj:`bool`: Flag indicating if mesh regions are used."""
        return self.__stub.GetUseMeshRegion(self.msg).value

    @use_mesh_region.setter
    def use_mesh_region(self, use_mesh_region):
        self.__stub.SetUseMeshRegion(messages.bool_property_message(self, use_mesh_region))

    @property
    def mesh_region_name(self):
        """:obj:`str`: Name of the mesh region to use."""
        return self.__stub.GetMeshRegionName(self.msg).value

    @mesh_region_name.setter
    def mesh_region_name(self, mesh_region_name):
        self.__stub.SetMeshRegionName(messages.string_property_message(self, mesh_region_name))


class HFSSPISolverSettings(SimulationSettingsBase):
    """Representis solver settings for HFSSPI simulations."""

    __stub: HFSSPISolverSettingsServiceStub = StubAccessor(StubType.hfss_pi_solver_sim_settings)

    @property
    def enhanced_low_frequency_accuracy(self):
        """:obj:`bool`: Flag indicating if enhanced low-frequency accuracy is enabled during simulation."""
        return self.__stub.GetEnhancedLowFrequencyAccuracy(self.msg).value

    @enhanced_low_frequency_accuracy.setter
    def enhanced_low_frequency_accuracy(self, enhanced_low_frequency_accuracy):
        self.__stub.SetEnhancedLowFrequencyAccuracy(
            messages.bool_property_message(self, enhanced_low_frequency_accuracy)
        )

    @property
    def via_area_cutoff_circ_elems(self) -> str:
        """:obj:`str`: Pwr/Gnd vias with an area smaller than this value are simplified during simulation."""
        return self.__stub.GetViaAreaCutoffCircElems(self.msg).value

    @via_area_cutoff_circ_elems.setter
    def via_area_cutoff_circ_elems(self, via_area_cutoff_circ_elems):
        self.__stub.SetViaAreaCutoffCircElems(
            messages.string_property_message(self, via_area_cutoff_circ_elems)
        )


class HFSSPIAdvancedSettings(SimulationSettingsBase):
    """Represents advanced settings for HFSSPI simulations."""

    __hfss_pi_stub: HFSSPIAdvancedSettingsServiceStub = StubAccessor(
        StubType.hfss_pi_advanced_sim_settings
    )
    __advanced_sim_settings_stub: AdvancedSettingsServiceStub = StubAccessor(
        StubType.advanced_sim_settings
    )
    __advanced_mesh_sim_settings_stub: AdvancedMeshingSettingsServiceStub = StubAccessor(
        StubType.advanced_mesh_sim_settings
    )

    @property
    def small_void_area(self):
        """:obj:`float`: Voids with an area smaller than this value are ignored during simulation."""
        return self.__advanced_sim_settings_stub.GetSmallVoidArea(self.msg).value

    @small_void_area.setter
    def small_void_area(self, small_void_area):
        self.__advanced_sim_settings_stub.SetSmallVoidArea(
            messages.double_property_message(self, small_void_area)
        )

    @property
    def small_plane_area(self) -> str:
        """:obj:`str`: Planes with an area smaller than this value are ignored during simulation."""
        return self.__hfss_pi_stub.GetSmallPlaneArea(self.msg).value

    @small_plane_area.setter
    def small_plane_area(self, small_plane_area: str):
        self.__hfss_pi_stub.SetSmallPlaneArea(
            messages.string_property_message(self, small_plane_area)
        )

    @property
    def remove_floating_geometry(self):
        """:obj:`bool`: Flag indicating if a geometry not connected to any other geometry is removed."""
        return self.__advanced_sim_settings_stub.GetRemoveFloatingGeometry(self.msg).value

    @remove_floating_geometry.setter
    def remove_floating_geometry(self, remove_floating_geometry):
        self.__advanced_sim_settings_stub.SetRemoveFloatingGeometry(
            messages.bool_property_message(self, remove_floating_geometry)
        )

    @property
    def zero_metal_layer_thickness(self) -> str:
        """:obj:`str`: Pwr/Gnd layers with a thickness smaller than this value are simplified during simulation."""
        return self.__hfss_pi_stub.GetZeroMetalLayerThickness(self.msg).value

    @zero_metal_layer_thickness.setter
    def zero_metal_layer_thickness(self, zero_metal_layer_thickness):
        self.__hfss_pi_stub.SetZeroMetalLayerThickness(
            messages.string_property_message(self, zero_metal_layer_thickness)
        )

    @property
    def auto_model_resolution(self):
        """:obj:`bool`: Flag indicating if model resolution is automatically calculated."""
        return self.__hfss_pi_stub.GetICModeAutoResolution(self.msg).value

    @auto_model_resolution.setter
    def auto_model_resolution(self, auto_model_resolution):
        self.__hfss_pi_stub.SetICModeAutoResolution(
            messages.bool_property_message(self, auto_model_resolution)
        )

    @property
    def model_resolution_length(self):
        """:obj:`str`: Model resolution to use when manually setting the model resolution."""
        return self.__hfss_pi_stub.GetICModeLength(self.msg).value

    @model_resolution_length.setter
    def model_resolution_length(self, model_resolution_length):
        self.__hfss_pi_stub.SetICModeLength(
            messages.string_property_message(self, model_resolution_length)
        )

    @property
    def max_num_arc_points(self):
        """:obj:`str`: Maximum number of points used to approximate arcs."""
        return self.__advanced_mesh_sim_settings_stub.GetMaxNumArcPoints(self.msg).value

    @max_num_arc_points.setter
    def max_num_arc_points(self, max_num_arc_points):
        self.__advanced_mesh_sim_settings_stub.SetMaxNumArcPoints(
            messages.uint64_property_message(self, max_num_arc_points)
        )

    @property
    def use_arc_chord_error_approx(self):
        """:obj:`bool`: Flag indicating if arc chord error approximation is used."""
        return self.__advanced_mesh_sim_settings_stub.GetUseArcChordErrorApprox(self.msg).value

    @use_arc_chord_error_approx.setter
    def use_arc_chord_error_approx(self, use_arc_chord_error_approx):
        self.__advanced_mesh_sim_settings_stub.SetUseArcChordErrorApprox(
            messages.bool_property_message(self, use_arc_chord_error_approx)
        )

    @property
    def arc_to_chord_error(self):
        """:obj:`str`: Maximum allowable arc to chord error."""
        return self.__advanced_mesh_sim_settings_stub.GetArcChordErrorApprox(self.msg).value

    @arc_to_chord_error.setter
    def arc_to_chord_error(self, arc_to_chord_error):
        self.__advanced_mesh_sim_settings_stub.SetArcChordErrorApprox(
            messages.string_property_message(self, arc_to_chord_error)
        )

    @property
    def num_via_sides(self):
        """:obj:`int`: Number of sides a via is considered to have."""
        return self.__stub.GetNumViaSides(self.msg).value

    @num_via_sides.setter
    def num_via_sides(self, num_via_sides):
        self.__stub.SetNumViaSides(messages.uint64_property_message(self, num_via_sides))

    @property
    def mesh_for_via_plating(self):
        """:obj:`bool`: Flag indicating if meshing for via plating is enabled."""
        return self.__stub.GetMeshForViaPlating(self.msg).value

    @mesh_for_via_plating.setter
    def mesh_for_via_plating(self, mesh_for_via_plating):
        self.__stub.SetMeshForViaPlating(messages.bool_property_message(self, mesh_for_via_plating))

    @property
    def via_material(self):
        """:obj:`str`: Default via material."""
        return self.__stub.GetViaMaterial(self.msg).value

    @via_material.setter
    def via_material(self, via_material):
        self.__stub.SetViaMaterial(messages.string_property_message(self, via_material))



"""Technology definition."""
from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ansys.edb.core.database import Database

import ansys.api.edb.v1.technology_def_pb2 as pb

from ansys.edb.core.inner import ObjBase, messages
from ansys.edb.core.session import StubAccessor, StubType, TechnologyDefServiceStub


class TechnologyDef(ObjBase):
    """Represents a technology."""

    __stub: TechnologyDefServiceStub = StubAccessor(StubType.technology)

    @classmethod
    def create(
        cls,
        database: Database,
        definition_name: str,
        technology_file: str,
        gfd_file: str = "",
        layer_filter_file: str = "",
        create_backplane: bool = False,
    ) -> TechnologyDef:
        """Create a technology definition in a given database.

        Parameters
        ----------
        database : .Database
            Database to create the technology definition in.
        definition_name : str
            Name of the technology definition.
        technology_file: str
            Path of the technology file.
        gfd_file: str
            Path of the GFD file.
        layer_filter_file: str
            Path of the layer filter file.
        create_backplane: bool
            Determines whether to create a backplane or not.

        Returns
        -------
        .TechnologyDef
        """
        return TechnologyDef(
            cls.__stub.Create(
                pb.TechnologyDefCreateMessage(
                    database=messages.edb_obj_message(database),
                    definition_name=definition_name,
                    vlc_tech_file=technology_file,
                    gfd_file=gfd_file,
                    layer_filter_file=layer_filter_file,
                    create_backplane=create_backplane,
                )
            )
        )

    @classmethod
    def find_by_name(cls, database: Database, definition_name: str) -> TechnologyDef:
        """Find a technology definition by name in a given database.

        Parameters
        ----------
        database : .Database
            Database to search for the technology definition.
        definition_name : str
            Name of the technology definition.

        Returns
        -------
        .TechnologyDef
            Technology definition object found. \
            If a technology definition isn't found, the returned technology definition is :meth:`null <.is_null>`.
        """
        return TechnologyDef(
            cls.__stub.FindByName(messages.edb_obj_name_message(database, definition_name))
        )

    def delete(self):
        """Delete the technology definition."""
        msg = messages.edb_obj_message(self)
        self.__stub.Delete(msg)

    @property
    def tech_file(self) -> str:
        """:obj:`str`: The technology file for this technology definition.

        This property is read-only.
        """
        return self.__stub.GetTechFile(self.msg).value

    @property
    def gfd_file(self) -> str:
        """:obj:`str`: The GFD file for this technology definition.

        This property is read-only.
        """
        return self.__stub.GetGFDFile(self.msg).value

    @property
    def layer_filter_file(self) -> str:
        """:obj:`str`: The layer filter file for this technology definition.

        This property is read-only.
        """
        return self.__stub.GetLayerFile(self.msg).value

    @property
    def name(self) -> str:
        """:obj:`str`: Name of the technology definition.

        This property is read-only.
        """
        return self.__stub.GetName(self.msg).value

    @property
    def is_create_backplane(self) -> str:
        """:obj:`bool`: Whether the technology definition creates a backplane.

        This property is read-only.
        """
        return self.__stub.GetIsCreateBackplane(self.msg).value



import os
import tempfile

# configs are initialized with system environment variables. Can be manually overwritten.
configs = {**os.environ}


def server_exe_dir():
    """ANSYS EM Root directory where EDB_RPC_Server.exe is located."""
    if "ANSYSEM_EDB_EXE_DIR" in configs:
        return configs["ANSYSEM_EDB_EXE_DIR"]
    # TODO: add 'VERSION=23.2/24.1/etc' and look for system installation


def temp_dir():
    """Temporary directory. system default if unspecified."""
    if "ANSYSEM_EDB_TEMP_DIR" in configs:
        return configs["ANSYSEM_EDB_TEMP_DIR"]
    return tempfile.mkdtemp()



from typing import List

from utils.fixtures import *  # noqa
from utils.test_utils import create_edb_obj_collection_msg, equals

from ansys.edb.core import database as database
from ansys.edb.core.inner.messages import bool_message, empty_message, int64_message, str_message
from ansys.edb.core.layout.cell import Cell

# Helper fixtures and functions


@pytest.fixture
def db_obj(edb_obj_msg):
    """Fixture for creating a Database used during testing.

    Returns
    -------
    Database
    """
    return database.Database(edb_obj_msg)


def _patch_database_stub(mocked_stub, test_method_name, expected_response):
    """Helper method that patches the given Database stub method with a mock server.

    Parameters
    ----------
    mocked_stub : unittest.mock.Mock
    test_method_name : str
    expected_response : Any

    Returns
    -------
    unittest.mock.Mock
    """
    mock = getattr(mocked_stub(database, database.Database), test_method_name)
    mock.return_value = expected_response
    return mock


# Tests


def test_create(random_str, edb_obj_msg, mocked_stub):
    """Test for the Database.create(db_path) method

    Parameters
    ----------
    random_str : str
        String to be used as the db_path parameter
    edb_obj_msg : EDBObjMessage
        The message that will be expected as the response from the mock server
    mocked_stub : unittest.mock.Mock
        Mocker used to patch primitive stub with the mock server
    """
    mock_server = _patch_database_stub(mocked_stub, "Create", edb_obj_msg)

    created_db = database.Database.create(random_str)

    mock_server.assert_called_once_with(str_message(random_str))

    assert isinstance(created_db, database.Database)
    assert equals(created_db.msg, edb_obj_msg)


def test_open(random_str, bool_val, edb_obj_msg, mocked_stub):
    """Test for the Database.open(db_path, read_only) method

    Parameters
    ----------
    random_str : str
        String to be used as the db_path parameter
    bool_val : bool
        Boolean to be used as the read_only parameter
    mocked_stub : unittest.mock.Mock
        Mocker used to patch primitive stub with the mock server
    """
    mock_server = _patch_database_stub(mocked_stub, "Open", edb_obj_msg)

    opened_db = database.Database.open(random_str, bool_val)

    mock_server.assert_called_once_with(
        database.database_pb2.OpenDatabaseMessage(
            edb_path=random_str,
            read_only=bool_val,
        )
    )

    assert isinstance(opened_db, database.Database)
    assert equals(opened_db.msg, edb_obj_msg)


def test_delete(random_str, bool_val, mocked_stub):
    """Test for the Database.delete(db_path) method

    Parameters
    ----------
    random_str : str
        String to be used as the db_path parameter
    bool_val : bool
        The expected return value
    mocked_stub : unittest.mock.Mock
        Mocker used to patch primitive stub with the mock server
    """
    mock_server = _patch_database_stub(mocked_stub, "Delete", empty_message())

    result = database.Database.delete(random_str)

    mock_server.assert_called_once_with(str_message(random_str))

    assert result is None


def test_save(db_obj, bool_val, mocked_stub):
    """Test for the Database.save() method

    Parameters
    ----------
    db_obj : Database
        Database to be saved
    bool_val : bool
        The expected return value
    mocked_stub : unittest.mock.Mock
        Mocker used to patch primitive stub with the mock server
    """
    mock_server = _patch_database_stub(mocked_stub, "Save", empty_message())

    result = db_obj.save()

    mock_server.assert_called_once_with(db_obj.msg)

    assert result is None


def test_close(db_obj, bool_val, mocked_stub):
    """Test for the Database.close() method

    Parameters
    ----------
    db_obj : Database
        Database to be closed
    bool_val : bool
        The expected return value
    mocked_stub : unittest.mock.Mock
        Mocker used to patch primitive stub with the mock server
    """
    mock_server = _patch_database_stub(mocked_stub, "Close", empty_message())

    expected_message = db_obj.msg
    result = db_obj.close()

    mock_server.assert_called_once_with(expected_message)

    assert result is None
    assert db_obj.is_null


@pytest.mark.parametrize("expected_num_top_cells", list(range(3)))
def test_top_circuit_cells(db_obj, expected_num_top_cells, mocked_stub):
    """Test for the Database.top_circuit_cells property

    Parameters
    ----------
    db_obj : Database
        Database to retrieve the top circuit cells of
    expected_num_top_cells : int
        The expected number of retrieved top cells
    mocked_stub : unittest.mock.Mock
        Mocker used to patch primitive stub with the mock server
    """
    expected_response = create_edb_obj_collection_msg(expected_num_top_cells)

    mock_server = _patch_database_stub(mocked_stub, "GetTopCircuits", expected_response)

    top_cells = db_obj.top_circuit_cells

    mock_server.assert_called_once_with(db_obj.msg)

    assert isinstance(top_cells, List)
    assert len(top_cells) == expected_num_top_cells
    for top_cell_idx in range(expected_num_top_cells):
        top_cell = top_cells[top_cell_idx]
        assert isinstance(top_cell, Cell)
        assert equals(top_cell.msg, expected_response.items[top_cell_idx])


def test_get_id(db_obj, random_int, mocked_stub):
    """Test for the Database.get_id() method

    Parameters
    ----------
    db_obj : Database
        Database to get the id of
    random_int : int
        The expected return value
    mocked_stub : unittest.mock.Mock
        Mocker used to patch primitive stub with the mock server
    """
    mock_server = _patch_database_stub(mocked_stub, "GetId", int64_message(random_int))

    db_id = db_obj.edb_uid

    mock_server.assert_called_once_with(db_obj.msg)

    assert db_id == random_int


def test_is_read_only(db_obj, bool_val, mocked_stub):
    """Test for the Database.is_read_only() method

    Parameters
    ----------
    db_obj : Database
        Database to get the "read only" flag of
    bool_val : bool
        The expected return value
    mocked_stub : unittest.mock.Mock
        Mocker used to patch primitive stub with the mock server
    """
    mock_server = _patch_database_stub(mocked_stub, "IsReadOnly", bool_message(bool_val))

    is_read_only = db_obj.is_read_only

    mock_server.assert_called_once_with(db_obj.msg)

    assert is_read_only == bool_val


def test_find_by_id(random_int, edb_obj_msg, mocked_stub):
    """Test for the Database.is_read_only(db_id) method

    Parameters
    ----------
    random_int : int
        Int to be used as the db_id parameter
    edb_obj_msg : EDBObjMessage
        The message that will be expected as the response from the mock server
    mocked_stub : unittest.mock.Mock
        Mocker used to patch primitive stub with the mock server
    """
    mock_server = _patch_database_stub(mocked_stub, "FindById", edb_obj_msg)

    found_db = database.Database.find_by_id(random_int)

    mock_server.assert_called_once_with(int64_message(random_int))

    assert isinstance(found_db, database.Database)
    assert equals(found_db.msg, edb_obj_msg)



import ansys.api.edb.v1.edb_messages_pb2 as edb_messages_pb2
from utils.fixtures import *  # noqa

import ansys.edb.core.geometry.point_data as point_data
import ansys.edb.core.utility.value as value


@pytest.mark.parametrize(
    "p2, expected",
    [
        [(1, 2), (2, 3)],
        [[1, 2], (2, 3)],
        [point_data.PointData((1, 2)), (2, 3)],
        [(1 + 2j, 3), (2 + 2j, 4)],
    ],
)
def test_addition(p2, expected):
    p1 = point_data.PointData((1, 1))
    p3 = p1 + p2
    assert p3.x.complex == expected[0]
    assert p3.y.complex == expected[1]


@pytest.mark.parametrize(
    "p2, expected",
    [
        [(1, 2), (4, 3)],
        [[1, 2], (4, 3)],
        [point_data.PointData((1, 2)), (4, 3)],
        [(1 + 2j, 3), (4 - 2j, 2)],
    ],
)
def test_subtraction(p2, expected):
    p1 = point_data.PointData((5, 5))
    p3 = p1 - p2
    assert p3.x.complex == expected[0]
    assert p3.y.complex == expected[1]


@pytest.mark.parametrize(
    "coord, expected",
    [
        [1, False],
        [(1, 2), False],
        [value.Value(1), False],
        [(value.Value(1), value.Value(2)), False],
        ["h", True],
        [("x", 2), True],
        [(1, "y"), True],
    ],
)
def test_is_parametric(mocked_stub, coord, expected):
    mock = mocked_stub(value, value.Value)
    mock.CreateValue.side_effect = lambda payload: edb_messages_pb2.ValueMessage(text=payload.text)
    p = point_data.PointData(coord)
    assert p.is_parametric == expected


@pytest.mark.parametrize("coord, magnitude", [[(3, 4), 5], [1, 0]])
def test_magnitude(coord, magnitude):
    p = point_data.PointData(coord)
    assert p.magnitude() == magnitude


@pytest.mark.parametrize("coord, normalized_coord", [[(3, 4), (3 / 5, 4 / 5)], [1, 0]])
def test_normalized(coord, normalized_coord):
    p = point_data.PointData(coord)
    assert p.normalized() == point_data.PointData(normalized_coord)


@pytest.mark.parametrize("coord1, coord2, dist", [[(1, 2), (4, 6), 5], [1, (1, 1), 0]])
def test_distance(coord1, coord2, dist):
    p1, p2 = point_data.PointData(coord1), point_data.PointData(coord2)
    assert p1.distance(p2) == dist



import pytest

from ansys.edb.core.layout.cell import Cell


@pytest.mark.parametrize(
    ("layer_name", "is_null"), (("L1", False), ("NonExistentLayer", True), ("l1", False))
)
def test_find_by_name(circuit_cell_with_stackup: Cell, is_null: bool, layer_name: str):
    """Test that finding a layer by name returns a correct layer object."""
    layer = circuit_cell_with_stackup.layout.layer_collection.find_by_name(layer_name)
    assert layer is not None and layer.is_null == is_null


def test_tsv_properties(circuit_cell_with_overlapping_stackup: Cell):
    """Test that finding a layer by name returns a correct layer object."""
    layer = circuit_cell_with_overlapping_stackup.layout.layer_collection.find_by_name("V1")
    assert layer is not None and not layer.is_null
    assert not layer.is_tsv
    layer.is_tsv = True
    layer.add_oxide_layers([("0.1um", "silicon_dioxide"), ("25nm", "FR4_epoxy")])
    assert layer.is_tsv
    assert len(layer.oxide_layers) == 2
    assert layer.oxide_layers == [("0.1um", "silicon_dioxide"), ("25nm", "FR4_epoxy")]



from ansys.edb.core.geometry.polygon_data import PolygonData
from ansys.edb.core.layout.cell import Cell
from ansys.edb.core.net.net import Net
from ansys.edb.core.primitive.path import Path as Line
from ansys.edb.core.primitive.path import PathCornerType, PathEndCapType
from ansys.edb.core.primitive.primitive import PrimitiveType
from ansys.edb.core.terminal.edge_terminal import EdgeTerminal, PrimitiveEdge
from ansys.edb.core.terminal.terminal import Terminal


def test_primitive_edge_primitive(circuit_cell_with_edge_terminals: Cell):
    layout = circuit_cell_with_edge_terminals.layout
    terminal: EdgeTerminal = Terminal.find(layout, "P1")
    assert len(terminal.edges) == 1
    assert terminal.edges[0].primitive.primitive_type == PrimitiveType.PATH


def test_set_edges(circuit_cell_with_edge_terminals: Cell):
    layout = circuit_cell_with_edge_terminals.layout

    line = Line.create(
        circuit_cell_with_edge_terminals.layout,
        "L1",
        Net.create(circuit_cell_with_edge_terminals.layout, "NET2"),
        0.2e-3,
        PathEndCapType.FLAT,
        PathEndCapType.FLAT,
        PathCornerType.ROUND,
        PolygonData([[-0.9e-3, -0.4e-3], [0.9e-3, -0.4e-3]]),
    )

    terminal: EdgeTerminal = Terminal.find(layout, "P1")
    assert len(terminal.edges) == 1

    edge1 = PrimitiveEdge.create(line, [-0.9e-3, -0.4e-3])
    assert terminal.edges[0].id != edge1.id
    terminal.edges = [edge1]
    assert terminal.edges[0].id == edge1.id



import random

from ansys.api.edb.v1.edb_messages_pb2 import EDBObjCollectionMessage, EDBObjMessage

from ansys.edb.core.inner import ObjBase

# Comparison utils


def equals(v0, v1):
    """Checks if two values are equivalent

    Returns
    -------
    bool
    """
    if isinstance(v0, EDBObjMessage) and isinstance(v1, EDBObjMessage):
        return v0.SerializeToString() == v1.SerializeToString()
    if isinstance(v0, ObjBase) and isinstance(v1, ObjBase):
        return equals(v0.msg, v1.msg)
    raise NotImplementedError()


def generate_random_int():
    """Generates a random integer between 0 and 100,000. Note this is not guaranteed to be unique.

    Returns
    -------
    int
    """
    return random.randint(0, 100000)


# Msg utils


def create_edb_obj_msg():
    """Creates and EDBObjMessage where the id is a random number

    Returns
    -------
    EDBObjMessage
    """
    return EDBObjMessage(id=generate_random_int())


def create_edb_obj_msgs(num_msgs):
    """Creates a list of EDBObjMessages

    Parameters
    ----------
    num_msgs : int
        number of EDBObjMessages to create

    Returns
    -------
    List[EDBObjMessage]
    """
    return [create_edb_obj_msg() for _ in range(num_msgs)]


def create_edb_obj_collection_msg(num_msgs):
    """Creates an EDBObjCollectionMessage

    Parameters
    ----------
    num_msgs : int
        number of EDBObjMessages to store in the EDBObjCollectionMessage

    Returns
    -------
    EDBObjCollectionMessage
    """
    return EDBObjCollectionMessage(items=create_edb_obj_msgs(num_msgs))


# Mock server utils


def patch_stub(stub_getter, mocker, test_method_name, expected_response, **kwargs):
    """Helper method that patches the given stub method.

    Parameters
    ----------
    stub_getter : str
        Module path to stub getter method
    mocker : pytest_mock.MockerFixture
        Mocker fixture used when patching the stub method
    test_method_name : str
        Name of stub method to be patched
    expected_response : Any
        The response message that will be returned by the mocked stub method
    Returns
    -------
    unittest.mock.Mock
    """

    # Create the mock server
    mock_server = mocker.Mock()
    mock_server_attr = {test_method_name + ".return_value": expected_response}
    mock_server.configure_mock(**mock_server_attr)

    # Patch the stub getter method with the mock server

    mocker.patch(stub_getter, return_value=mock_server, **kwargs)

    # Return the mock server
    return getattr(mock_server, test_method_name)



import settings

from ansys.edb.core.database import Database
from ansys.edb.core.layout.cell import Cell, CellType
from ansys.edb.core.layout.voltage_regulator import PowerModule, VoltageRegulator
from ansys.edb.core.session import launch_session


def test_power_module():
    session = launch_session(settings.server_exe_dir(), 50051)

    db = Database.create(settings.temp_dir())

    cell = Cell.create(db, CellType.CIRCUIT_CELL, "my_cell")
    layout = cell.layout

    vr1 = VoltageRegulator.create(layout, "VR1", True, 89, "10mA", 22)
    pm1_orig = PowerModule("group1", "p1", "n1", 18, True)
    pm2_orig = PowerModule("group3", "p1", "n0", 7, False)
    print(pm1_orig.comp_group_name)

    print("Adding two power modules; only one is active")
    vr1.add_power_module(pm1_orig)
    vr1.add_power_module(pm2_orig)
    n = vr1.num_power_modules
    print("number of power modules = " + str(n))
    na = vr1.num_active_power_modules
    print("number of active power modules = " + str(na))

    print("Getting a power module")
    pm1 = vr1.get_power_module(pm1_orig.comp_group_name)
    is_same = True
    if pm1.comp_group_name != pm1_orig.comp_group_name:
        is_same = False
    elif pm1.pos_output_terminal != pm1_orig.pos_output_terminal:
        is_same = False
    elif pm1.neg_output_terminal != pm1_orig.neg_output_terminal:
        is_same = False
    elif pm1.relative_strength != pm1_orig.relative_strength:
        is_same = False
    elif pm1.active != pm1_orig.active:
        is_same = False
    print("retrieved pm is same as original" if is_same else "retrieved pm is NOT same as original")

    print("Getting all power modules")
    all_pms = vr1.get_all_power_modules()
    txt = "retrieved modules ["
    add_comma = False
    for x in all_pms:
        if add_comma:
            txt += ","
        txt += x.comp_group_name
        add_comma = True
    print(txt + "]")

    print("Removing one power module")
    vr1.remove_power_module(pm1.comp_group_name)
    n = vr1.num_power_modules
    print("number of power modules = " + str(n))

    print("Removing all power modules")
    vr1.remove_all_power_modules()
    n = vr1.num_power_modules
    print("number of power modules = " + str(n))
    session.disconnect()


if __name__ == "__main__":
    test_power_module()



import math

from ansys.api.edb.v1 import edb_messages_pb2, point_data_pb2
from google.protobuf import wrappers_pb2
import pytest
from utils.fixtures import *  # noqa

from ansys.edb.core.geometry import arc_data, polygon_data
from ansys.edb.core.utility import value


@pytest.mark.parametrize(
    "args, kwargs, expect_call",
    [
        [[(1, 1), (2, 2)], {}, False],
        [[(1, 1), (2, 2)], {"height": 3}, False],
        [[(1, 1), (2, 2)], {"radius": 3, "direction": "cw", "is_big": False}, True],
        [[(1, 1), (2, 2)], {"center": (2, 1), "direction": "cw"}, True],
        [[(1, 1), (2, 2)], {"thru": (1.5, 1.5)}, True],
        [[(1, 1), (2, 2)], {}, False],
    ],
)
def test_height(mocked_stub, args, kwargs, expect_call):
    mock = mocked_stub(arc_data, arc_data.ArcData).GetHeight
    mock.return_value = wrappers_pb2.FloatValue(value=2)
    _ = arc_data.ArcData(*args, **kwargs).height

    if expect_call:
        mock.assert_called_once()
    else:
        mock.assert_not_called()


test_arcs = [
    arc_data.ArcData((1, 1), (2, 2)),
    arc_data.ArcData((1, 1), (1 - 1e-10, 1 + 1e-10)),
    arc_data.ArcData((1, 1), (1, 1), height=1e9),
    arc_data.ArcData((1, 1), (1 - 1e-8, 1 + 1e-8)),
]


@pytest.mark.parametrize(
    "arc, tol, is_segment", zip(test_arcs, [0, 1e-9, 1e-9, 1e-9], [True, True, False, True])
)
def test_is_segment(arc, tol, is_segment):
    assert arc.is_segment(tol) == is_segment


@pytest.mark.parametrize(
    "arc, tol, is_point", zip(test_arcs, [0, 1e-9, 1e-9, 1e-9], [False, True, False, False])
)
def test_is_point(arc, tol, is_point):
    assert arc.is_point(tol) == is_point


def test_center(mocked_stub):
    mock = mocked_stub(arc_data, arc_data.ArcData).GetCenter
    mock.return_value = point_data_pb2.PointMessage(
        x=edb_messages_pb2.ValueMessage(constant=edb_messages_pb2.ComplexMessage(real=2)),
        y=edb_messages_pb2.ValueMessage(constant=edb_messages_pb2.ComplexMessage(real=2)),
    )
    arc = arc_data.ArcData((2, 0), (0, 2), height=2.0)
    center = arc.center
    mock.assert_called_once()
    assert center.x == 2
    assert center.y == 2


def test_bbox(mocked_stub):
    mock = mocked_stub(arc_data, arc_data.ArcData)
    mock.GetBoundingBox.return_value = point_data_pb2.BoxMessage(
        lower_left=point_data_pb2.PointMessage(
            x=edb_messages_pb2.ValueMessage(constant=edb_messages_pb2.ComplexMessage(real=0)),
            y=edb_messages_pb2.ValueMessage(constant=edb_messages_pb2.ComplexMessage(real=0)),
        ),
        upper_right=point_data_pb2.PointMessage(
            x=edb_messages_pb2.ValueMessage(constant=edb_messages_pb2.ComplexMessage(real=3)),
            y=edb_messages_pb2.ValueMessage(constant=edb_messages_pb2.ComplexMessage(real=4)),
        ),
    )
    arc = arc_data.ArcData((0, 0), (3, 4))
    bbox = arc.bbox
    assert len(bbox) == 4
    assert bbox.points == polygon_data.PolygonData(lower_left=(0, 0), upper_right=(3, 4)).points


@pytest.mark.parametrize(
    "height, is_big", [[0, False], [2.4, False], [2.6, True], [-2.4, False], [-2.6, True]]
)
def test_is_big(height, is_big):
    arc = arc_data.ArcData(start=(0, 0), end=(3, 4), height=height)
    assert value.Value(arc.start.distance(arc.end)).equals(5, 1e-6)
    assert arc.is_big() == is_big


@pytest.mark.parametrize(
    "expect_call, height, arc, length",
    [
        [False, 0, arc_data.ArcData(start=(0, 0), end=(3, 4), height=0), 5],
        [
            True,
            1,
            arc_data.ArcData(start=(0, 0), end=(3, 4), radius=1, direction="cw", is_big=False),
            math.acos(1) / 4,
        ],
    ],
)
def test_length(mocked_stub, expect_call, height, arc, length):
    mock = mocked_stub(arc_data, arc_data.ArcData)
    mock.GetHeight.return_value = wrappers_pb2.FloatValue(value=1)
    mock.GetRadius.return_value = wrappers_pb2.FloatValue(value=2)
    mock.GetAngle.return_value = wrappers_pb2.FloatValue(value=math.acos(1) / 8)

    assert value.Value(arc.length).equals(length, 1e-6)
    if expect_call:
        mock.GetRadius.assert_called_once()
        mock.GetAngle.assert_called_once()
    else:
        mock.GetRadius.assert_not_called()
        mock.GetAngle.assert_not_called()


def test_angle(mocked_stub):
    mock = mocked_stub(arc_data, arc_data.ArcData)
    mock.GetAngle.return_value = wrappers_pb2.FloatValue(value=0.5)
    arc = arc_data.ArcData(start=(0, 0), end=(3, 4))

    assert arc.angle() == 0.5
    mock.GetAngle.assert_called_once()

    arc1 = arc_data.ArcData(start=(0, 0), end=(3, 4))
    arc2 = arc_data.ArcData(start=(0, 0), end=(-3, -4))
    arc1.angle(arc2)


@pytest.mark.parametrize(
    "arc, tangent",
    [
        [arc_data.ArcData(start=(0, 0), end=(3, 4)), (3, 4)],
        [arc_data.ArcData(start=(0, 0), end=(3, 4), height=-0.1), (-4, 3)],
        [arc_data.ArcData(start=(0, 0), end=(3, 4), height=+0.1), (4, -3)],
    ],
)
def test_tangent_at(mocked_stub, arc, tangent):
    mock = mocked_stub(arc_data, arc_data.ArcData)
    mock.GetCenter.return_value = point_data_pb2.PointMessage(
        x=edb_messages_pb2.ValueMessage(constant=edb_messages_pb2.ComplexMessage(real=0)),
        y=edb_messages_pb2.ValueMessage(constant=edb_messages_pb2.ComplexMessage(real=0)),
    )
    point = (3, 4)
    assert arc.tangent_at(point) == tangent



"""Touchstone Settings."""

from enum import Enum, auto


class PassivityAlgorithm(Enum):
    """Enum representing passivity algorithm types."""

    CONVEX_OPTIMIZATION = auto()
    PASSIVITY_BY_PERTURBATION = auto()
    ITERATED_FITTING_OF_PV = auto()
    ITERATED_FITTING_OF_PVLF = auto()
    DONT_ENFORCE_PASSIVITY = auto()


class ColumnFittingAlgorithm(Enum):
    """Enum representing column fitting algorithm types."""

    COLUMN_ATA_TIME = auto()
    ENTRY_ATA_TIME = auto()
    WHOLE_MATRIX_ATA_TIME = auto()


class SSFittingAlgorithm(Enum):
    """Enum representing state space fitting algorithm types."""

    TWA = auto()
    ITERATIVE_RATIONAL = auto()
    FAST_FIT = auto()
    AUTO_FAST = auto()
    AUTO_ACCURATE = auto()


class ExportType(Enum):
    """Enum representing exported touchstone file types."""

    HFSS_TO_TOUCHSTONE_1_0 = 7
    HFSS_TO_TOUCHSTONE_2_0 = 8


class TouchstoneExportSettings:
    """Represents settings used when exporting touchstone files."""

    def __init__(
        self,
        export_after_solve: bool = False,
        export_dir: str = "",
        export_type: ExportType = ExportType.HFSS_TO_TOUCHSTONE_1_0,
        enforce_passivity: bool = False,
        enforce_causality: bool = False,
        use_common_ground: bool = True,
        show_gamma_comments: bool = False,
        do_renormalize: bool = True,
        renorm_impedance: float = 50.0,
        fitting_error: float = 0.5,
        max_poles: int = 10000,
        passivity_algorithm: PassivityAlgorithm = PassivityAlgorithm.ITERATED_FITTING_OF_PV,
        column_fitting_algorithm: ColumnFittingAlgorithm = ColumnFittingAlgorithm.WHOLE_MATRIX_ATA_TIME,
        ss_fitting_algorithm: SSFittingAlgorithm = SSFittingAlgorithm.FAST_FIT,
        relative_error_tolerance: bool = False,
        ensure_accurate_z_fit: bool = True,
        touchstone_format: str = "MA",
        touchstone_units: str = "GHz",
        touchstone_precision: int = 15,
    ):
        """
        Initialize a touchstone export settings object.

        Attributes
        ----------
        export_after_solve : bool, default: False
        export_dir : str, default = ""
        export_type : ExportType, default: ExportType.HFSS_TO_TOUCHSTONE_1_0
        enforce_passivity : bool, default: False
        enforce_causality : bool, default: False
        use_common_ground : bool, default: True
        show_gamma_comments : bool, default: False
        do_renormalize : bool, default: True
        renorm_impedance : float, default: 50.0
        fitting_error : float, default: 0.5
        max_poles : int, default: 10000
        passivity_algorithm : .PassivityAlgorithm, default: .PassivityAlgorithm.ITERATED_FITTING_OF_PV
        column_fitting_algorithm : .ColumnFittingAlgorithm, default: .ColumnFittingAlgorithm.WHOLE_MATRIX_ATA_TIME
        ss_fitting_algorithm : .SSFittingAlgorithm, default: .SSFittingAlgorithm.FAST_FIT
        relative_error_tolerance : bool, default: False
        ensure_accurate_z_fit : bool, default True
        touchstone_format : str, default: "MA"
        touchstone_units : str, default: "GHz"
        touchstone_precision : int, default: 15
        """
        self.export_after_solve = export_after_solve
        self.export_dir = export_dir
        self.export_type = export_type
        self.enforce_passivity = enforce_passivity
        self.enforce_causality = enforce_causality
        self.use_common_ground = use_common_ground
        self.show_gamma_comments = show_gamma_comments
        self.do_renormalize = do_renormalize
        self.renorm_impedance = renorm_impedance
        self.fitting_error = fitting_error
        self.max_poles = max_poles
        self.passivity_algorithm = passivity_algorithm
        self.column_fitting_algorithm = column_fitting_algorithm
        self.ss_fitting_algorithm = ss_fitting_algorithm
        self.relative_error_tolerance = relative_error_tolerance
        self.ensure_accurate_z_fit = ensure_accurate_z_fit
        self.touchstone_format = touchstone_format
        self.touchstone_units = touchstone_units
        self.touchstone_precision = touchstone_precision



"""ansys_edb_core_preload.py — installed to site-packages via a .pth hook.

This module is executed automatically by Python's site.py at interpreter
startup (before any user code runs) via ansys_edb_core_preload.pth.

On Linux, libEDB_RPC_Services.so requires a newer libstdc++ than the one
shipped with RHEL 8 / CentOS 8 and other enterprise distributions.  The
dynamic linker will reuse whichever libstdc++.so.6 is resident in the
process when libEDB_RPC_Services.so is dlopen'd.  If the system version was
loaded first (e.g. via grpcio's cygrpc extension), the newer GLIBCXX symbols
that Ansys requires are missing and initialization fails.

The fix: load the AnsysEM-bundled libstdc++.so.6 *here*, with RTLD_GLOBAL, so
it is the first copy resident in the process.  All subsequent C extensions
that request libstdc++.so.6 (grpcio, etc.) will find it already loaded and
use the same, newer copy.

Install directory resolution order:
  1. ANSYS_EDB_CORE_INSTALL_DIR — explicit override, highest priority.
  2. ANSYSEM_ROOT<XYZ> (e.g. ANSYSEM_ROOT261) — the variable with the largest
     numeric suffix is used when ANSYS_EDB_CORE_INSTALL_DIR is absent.
  3. If neither is set, a warning is printed and the system libstdc++ is used.

This is a no-op when:
  - Running on Windows or macOS (no action needed).
  - No Ansys install directory can be resolved (see above).
  - The AnsysEM libstdc++.so.6 is not present at the resolved path.
  - libstdc++.so.6 is already loaded (guards against double-execution).
"""
import ctypes
import os
import re
import sys

_YELLOW = "\033[93m"
_RESET = "\033[0m"
_PREFIX = "ansys-edb-core:"


def _warn(msg):
    print(f"{_YELLOW}{_PREFIX} {msg}{_RESET}", file=sys.stderr)


def _resolve_install_dir():
    """Return the AnsysEM install directory to use, or an empty string."""
    explicit = os.environ.get("ANSYS_EDB_CORE_INSTALL_DIR", "")
    if explicit:
        return explicit

    # Scan for ANSYSEM_ROOT<digits> variables and pick the one with the
    # largest numeric suffix.
    best_version = -1
    best_dir = ""
    for key, value in os.environ.items():
        m = re.fullmatch(r"ANSYSEM_ROOT(\d+)", key)
        if m:
            version = int(m.group(1))
            if version > best_version:
                best_version = version
                best_dir = value
    return best_dir


if sys.platform != "win32":
    _ansysem_dir = _resolve_install_dir()

    if not _ansysem_dir:
        _warn(
            "No AnsysEM install directory found. The system libstdc++.so.6 will "
            "be used, which may be incompatible with libEDB_RPC_Services.so. "
            "Set ANSYS_EDB_CORE_INSTALL_DIR or ANSYSEM_ROOT<XYZ> to the AnsysEM "
            " install directory (e.g. /path/to/AnsysEM) to ensure the correct library is loaded."
        )
    else:
        _libstdcpp = os.path.join(_ansysem_dir, "libstdc++.so.6")
        if os.path.isfile(_libstdcpp):
            # Check whether any libstdc++.so.6 is already mapped in this
            # process.  If it is, loading ours at this point would be too
            # late — the existing version is already locked in.
            _already_loaded = False
            try:
                with open("/proc/self/maps") as _maps:
                    for _line in _maps:
                        if "libstdc++" in _line:
                            _already_loaded = True
                            break
            except OSError:
                pass

            if not _already_loaded:
                try:
                    ctypes.CDLL(_libstdcpp, mode=ctypes.RTLD_GLOBAL)
                    _warn(f"Pre-loaded AnsysEM libstdc++.so.6 from '{_ansysem_dir}'.")
                except OSError:
                    pass



from time import time

import settings

from ansys.edb.core.database import Database
from ansys.edb.core.layer.layer import LayerType
from ansys.edb.core.layer.stackup_layer import StackupLayer
from ansys.edb.core.layout.cell import Cell, CellType
from ansys.edb.core.layout.layout import Layout
from ansys.edb.core.net.net import Net
from ansys.edb.core.primitive.rectangle import Rectangle, RectangleRepresentationType
from ansys.edb.core.session import session
from ansys.edb.core.utility.io_manager import IOMangementType, enable_io_manager
import tests.e2e.settings as settings

db_name = "io_performance_test.aedb"
lyr_name = "signal_1"
net_name = "net_1"
db = None
lyt: Layout = None


def query_prims():
    prims = lyt.primitives
    for prim in prims:
        prim.get_hfss_prop
        prim.polygon_data
        prim.net
        prim.layer
        prim.is_void
        prim.layout
    return len(prims)


def create_prims():
    num_prims = 1000
    for i in range(num_prims):
        rect = Rectangle.create(
            lyt,
            lyr_name,
            net_name,
            RectangleRepresentationType.CENTER_WIDTH_HEIGHT,
            0,
            0,
            1e-3,
            1e-3,
            0.0,
            0.0,
        )
        rect.set_hfss_prop("copper", True)
    return num_prims


def read_test():
    read_test_start = time()
    with enable_io_manager(IOMangementType.READ):
        num_read_test_queried_prims = query_prims()
    read_test_end = time()
    print(
        f"read_test time: {read_test_end - read_test_start} seconds (queried {num_read_test_queried_prims} primitives)"
    )
    return num_read_test_queried_prims


def write_test():
    write_test_start = time()
    with enable_io_manager(IOMangementType.WRITE):
        num_write_test_created_prims = create_prims()
    write_test_end = time()
    print(
        f"write_test time: {write_test_end - write_test_start} seconds "
        f"(created {num_write_test_created_prims} primitives)"
    )
    return num_write_test_created_prims


def read_and_write_test():
    read_write_test_start = time()
    with enable_io_manager(IOMangementType.READ_AND_WRITE):
        num_read_write_test_created_prims = create_prims()
        num_read_write_test_queried_prims = query_prims()
    read_write_test_end = time()
    print(
        f"read_and_write_test time: {read_write_test_end - read_write_test_start} seconds "
        f"(created {num_read_write_test_created_prims} primitives, queried {num_read_write_test_queried_prims} "
        f"primitives)"
    )
    return num_read_write_test_created_prims, num_read_write_test_queried_prims


def setup():
    global db, lyt

    db = Database.create(db_name)
    cell = Cell.create(db, CellType.CIRCUIT_CELL, "EMDesign1")
    lyt = cell.layout
    lyt.layer_collection.add_layers(
        [StackupLayer.create(lyr_name, LayerType.SIGNAL_LAYER, 1e-3, 0, "copper")]
    )
    Net.create(lyt, net_name)


def teardown():
    db.close()
    Database.delete(db_name)


def caching_flushing_blocking_test():
    blocking_start = time()
    with enable_io_manager(
        IOMangementType.READ_AND_WRITE | IOMangementType.NO_CACHE_INVALIDATION_NO_BUFFER_FLUSHING
    ):
        for prim in lyt.primitives:
            rect: Rectangle = prim
            geom = rect.polygon_data
            ll = geom.points[3]
            ur = geom.points[1]
            scaled_geom = geom.scale(2, (((ur.x - ll.x) / 2) + ll.x, ((ur.y - ll.y) / 2) + ll.y))
            scaled_ll = scaled_geom.points[3]
            scaled_ur = scaled_geom.points[1]
            rect.set_parameters(
                RectangleRepresentationType.LOWER_LEFT_UPPER_RIGHT,
                scaled_ll.x,
                scaled_ll.y,
                scaled_ur.x,
                scaled_ur.y,
                0,
                0,
            )
    blocking_end = time()
    print(f"total blocking time: {blocking_end - blocking_start} seconds")


def read_write_test():
    rw_start = time()
    num_created_prims = write_test()
    num_queried_prims = read_test()
    prim_io_op_counts = read_and_write_test()
    rw_end = time()
    total_created_prims = num_created_prims + prim_io_op_counts[0]
    total_queried_prims = num_queried_prims + prim_io_op_counts[1]
    print(
        f"total rw time: {rw_end - rw_start} seconds (created {total_queried_prims} primitives, "
        f"queried {total_created_prims} primitives)"
    )


if __name__ == "__main__":
    with session(settings.server_exe_dir()):
        setup()
        start = time()
        read_write_test()
        caching_flushing_blocking_test()
        end = time()
        print(f"total time: {end - start} seconds")
        teardown()



import settings

from ansys.edb.core.database import Database
from ansys.edb.core.layout.cell import Cell, CellType
from ansys.edb.core.session import session
from ansys.edb.core.simulation_setup.adaptive_solutions import (
    BroadbandAdaptiveSolution,
    MultiFrequencyAdaptiveSolution,
    SingleFrequencyAdaptiveSolution,
)
from ansys.edb.core.simulation_setup.hfss_simulation_settings import (
    AdaptType,
    BasisFunctionOrder,
    HFSSAdvancedMeshingSettings,
    HFSSAdvancedSettings,
    HFSSDCRSettings,
    HFSSGeneralSettings,
    HFSSSettingsOptions,
    HFSSSimulationSettings,
    HFSSSolverSettings,
    SolverType,
)
from ansys.edb.core.simulation_setup.hfss_simulation_setup import HfssSimulationSetup
from ansys.edb.core.simulation_setup.mesh_operation import (
    LengthMeshOperation,
    SkinDepthMeshOperation,
)
from ansys.edb.core.simulation_setup.simulation_settings import ViaStyle
from ansys.edb.core.simulation_setup.simulation_setup import SweepData


def do_hfss_sim_setup_test(hfss_sim_setup):
    og_name = hfss_sim_setup.name
    hfss_sim_setup.name = "new_name"
    new_name = hfss_sim_setup.name

    og_pos = hfss_sim_setup.position
    hfss_sim_setup.position = 1
    new_pos = hfss_sim_setup.position

    og_sweep_data = hfss_sim_setup.sweep_data
    hfss_sim_setup.sweep_data = [SweepData("sweep_1", "LIN", "1GHz", "10GHz", "1GHz")]
    new_sweep_data = hfss_sim_setup.sweep_data

    og_mesh_ops = hfss_sim_setup.mesh_operations
    sk_op = SkinDepthMeshOperation(
        name="sk_op", mesh_region="mes_region", net_layer_info=[("net", "lyr", True)]
    )
    l_op = LengthMeshOperation(
        name="l_op", mesh_region="mes_region", net_layer_info=[("net", "lyr", True)]
    )
    hfss_sim_setup.mesh_operations = [sk_op, l_op]
    new_mesh_ops = hfss_sim_setup.mesh_operations

    print("done")


def do_general_test(settings: HFSSGeneralSettings):
    og_sas = settings.single_frequency_adaptive_solution
    single_sol = SingleFrequencyAdaptiveSolution(use_mx_conv_data=True)
    mx_data = single_sol.mx_conv_data
    mx_data.add_entry("port0", "port1", 1.0, 2.0)
    mx_data.add_entry("port0", "port1", 1.0, 3.0)
    single_sol.mx_conv_data = mx_data
    settings.single_frequency_adaptive_solution = single_sol
    new_sas = settings.single_frequency_adaptive_solution

    og_m = settings.multi_frequency_adaptive_solution
    settings.multi_frequency_adaptive_solution = MultiFrequencyAdaptiveSolution()
    new_m = settings.multi_frequency_adaptive_solution

    og_b = settings.broadband_adaptive_solution
    settings.broadband_adaptive_solution = BroadbandAdaptiveSolution()
    new_b = settings.broadband_adaptive_solution

    og_save_fields = settings.save_fields
    settings.save_fields = not og_save_fields
    new_save_fields = settings.save_fields

    og_save_rad_fields = settings.save_rad_fields_only
    settings.save_rad_fields_only = not og_save_rad_fields
    new_save_rad_fields = settings.save_rad_fields_only

    og_adapt_type = settings.adaptive_solution_type
    settings.adaptive_solution_type = AdaptType.BROADBAND
    new_adapt_type = settings.adaptive_solution_type

    og_use_mesh_Region = settings.use_mesh_region
    settings.use_mesh_region = not og_use_mesh_Region
    new_use_mesh_region = settings.use_mesh_region

    og_mesh_region_name = settings.mesh_region_name
    settings.mesh_region_name = "new_name"
    new_mesh_region_name = settings.mesh_region_name

    og_use_parellel_refinement = settings.use_parallel_refinement
    settings.use_parallel_refinement = not og_use_parellel_refinement
    new_use_parallel_refinement = settings.use_parallel_refinement

    print("done")


def do_advanced_test(settings: HFSSAdvancedSettings):
    og_union_polys = settings.union_polygons
    settings.union_polygons = not og_union_polys
    new_union_polys = settings.union_polygons

    og_remove_floating_geometry = settings.remove_floating_geometry
    settings.remove_floating_geometry = not og_remove_floating_geometry
    new_remove_floating_geometry = settings.remove_floating_geometry

    og_small_void_area = settings.small_void_area
    settings.small_void_area = og_small_void_area + 1e-3
    new_small_void_area = settings.small_void_area

    og_use_defeature = settings.use_defeature
    settings.use_defeature = not og_use_defeature
    new_use_defeature = settings.use_defeature

    og_use_defeature_absolute_length = settings.use_defeature_absolute_length
    settings.use_defeature_absolute_length = not og_use_defeature_absolute_length
    new_use_defeature_absolute_length = settings.use_defeature_absolute_length

    og_defeature_absolute_length = settings.defeature_absolute_length
    settings.defeature_absolute_length = "1mm"
    new_defeature_absolute_length = settings.defeature_absolute_length

    og_defeature_ratio = settings.defeature_ratio
    settings.defeature_ratio = og_defeature_ratio + 1
    new_defeature_ratio = settings.defeature_ratio

    og_via_model_type = settings.via_model_type
    settings.via_model_type = ViaStyle.RIBBON
    new_via_model_type = settings.via_model_type

    og_num_via_density = settings.num_via_density
    settings.num_via_density = og_num_via_density + 1
    new_num_via_density = settings.num_via_density

    og_via_material = settings.via_material
    settings.via_material = "new_via_material"
    new_via_material = settings.via_material

    og_mesh_via_plating = settings.mesh_for_via_plating
    settings.mesh_for_via_plating = not og_mesh_via_plating
    new_mesh_via_plating = settings.mesh_for_via_plating

    og_ic_mode_auto_resolution = settings.ic_mode_auto_resolution
    settings.ic_mode_auto_resolution = not og_ic_mode_auto_resolution
    new_ic_mode_auto_resolution = settings.ic_mode_auto_resolution

    og_ic_mode_length = settings.ic_mode_length
    settings.ic_mode_length = "1mm"
    new_ic_mode_length = settings.ic_mode_length

    print("done")


def do_advanced_meshing_test(settings: HFSSAdvancedMeshingSettings):
    og_arc_step_size = settings.arc_step_size
    settings.arc_step_size = "1deg"
    new_arc_step_size = settings.arc_step_size

    og_circle_start_azimuth = settings.circle_start_azimuth
    settings.circle_start_azimuth = "1deg"
    new_circle_start_azimuth = settings.circle_start_azimuth

    og_max_num_arc_points = settings.max_num_arc_points
    settings.max_num_arc_points = 20
    new_max_num_arc_points = settings.max_num_arc_points

    og_use_arc_chord_error_approx = settings.use_arc_chord_error_approx
    settings.use_arc_chord_error_approx = not og_use_arc_chord_error_approx
    new_use_arc_chord_error_approx = settings.use_arc_chord_error_approx

    og_arc_to_chord_error = settings.arc_to_chord_error
    settings.arc_to_chord_error = "1mm"
    new_arc_to_chord_error = settings.arc_to_chord_error

    og_layer_snap_tol = settings.layer_snap_tol
    settings.layer_snap_tol = "1mm"
    new_layer_snap_tol = settings.layer_snap_tol

    print("done")


def do_solver_test(settings: HFSSSolverSettings):
    og_thin_signal_layer_threshold = settings.thin_signal_layer_threshold
    settings.thin_signal_layer_threshold = "1mm"
    new_thin_signal_layer_threshold = settings.thin_signal_layer_threshold

    og_thin_dielectric_layer_threshold = settings.thin_dielectric_layer_threshold
    settings.thin_dielectric_layer_threshold = "2mm"
    new_thin_dielectric_layer_threshold = settings.thin_dielectric_layer_threshold

    og_max_delta_z0 = settings.max_delta_z0
    settings.max_delta_z0 = og_max_delta_z0 + 1e-3
    new_max_delta_z0 = settings.max_delta_z0

    og_set_triangles_for_wave_port = settings.set_triangles_for_wave_port
    settings.set_triangles_for_wave_port = not og_set_triangles_for_wave_port
    new_set_triangles_for_wave_port = settings.set_triangles_for_wave_port

    og_min_triangles_for_wave_port = settings.min_triangles_for_wave_port
    settings.min_triangles_for_wave_port = og_min_triangles_for_wave_port + 1
    new_min_triangles_for_wave_port = settings.min_triangles_for_wave_port

    og_max_triangles_for_wave_port = settings.max_triangles_for_wave_port
    settings.max_triangles_for_wave_port = og_max_triangles_for_wave_port + 1
    new_max_triangles_for_wave_port = settings.max_triangles_for_wave_port

    og_enable_intra_plane_coupling = settings.enable_intra_plane_coupling
    settings.enable_intra_plane_coupling = not og_enable_intra_plane_coupling
    new_enable_intra_plane_coupling = settings.enable_intra_plane_coupling

    print("done")


def do_dcr_test(settings: HFSSDCRSettings):
    og_max_passes = settings.max_passes
    settings.max_passes = og_max_passes + 1
    new_max_passes = settings.max_passes

    og_min_passes = settings.min_passes
    settings.min_passes = og_min_passes + 2
    new_min_passes = settings.min_passes

    og_min_converged_passes = settings.min_converged_passes
    settings.min_converged_passes = og_min_converged_passes + 3
    new_min_converged_passes = settings.min_converged_passes

    og_percent_error = settings.percent_error
    settings.percent_error = og_percent_error + 0.5
    new_percent_error = settings.percent_error

    og_percent_refinement_per_pass = settings.percent_refinement_per_pass
    settings.percent_refinement_per_pass = og_percent_refinement_per_pass + 0.75
    new_percent_refinement_per_pass = settings.percent_refinement_per_pass

    print("done")


def do_options_test(settings: HFSSSettingsOptions):
    og_do_lamda_refine = settings.do_lambda_refine
    settings.do_lambda_refine = not og_do_lamda_refine
    new_do_lamda_refine = settings.do_lambda_refine

    og_lamda_target = settings.lambda_target
    settings.lambda_target = og_lamda_target + 0.75
    new_lamda_target = settings.lambda_target
    og_use_default_lamda_value = settings.use_default_lambda_value
    settings.use_default_lambda_value = not og_use_default_lamda_value
    new_use_default_lamda_value = settings.use_default_lambda_value

    og_max_refinement_per_pass = settings.max_refinement_per_pass
    settings.max_refinement_per_pass = og_max_refinement_per_pass + 1
    new_max_refinement_per_pass = settings.max_refinement_per_pass

    og_min_passes = settings.min_passes
    settings.min_passes = og_min_passes + 1
    new_min_passes = settings.min_passes

    og_min_converged_passes = settings.min_converged_passes
    settings.min_converged_passes = og_min_converged_passes + 1
    new_min_converged_passes = settings.min_converged_passes

    og_use_max_refinement = settings.use_max_refinement
    settings.use_max_refinement = not og_use_max_refinement
    new_use_max_refinement = settings.use_max_refinement

    # TODO: fix this
    og_order_basis = settings.order_basis
    settings.order_basis = BasisFunctionOrder.MIXED_ORDER
    new_order_basis = settings.order_basis

    og_order_basis_2 = settings.order_basis
    settings.order_basis = BasisFunctionOrder.ZERO_ORDER
    new_order_basis_2 = settings.order_basis

    og_solver_type = settings.solver_type
    settings.solver_type = SolverType.ITERATIVE_SOLVER
    new_solver_type = settings.solver_type

    og_relative_residual = settings.relative_residual
    settings.relative_residual = og_relative_residual + 0.75
    new_relative_residual = settings.relative_residual

    og_enhanced_low_frequency_accuracy = settings.enhanced_low_frequency_accuracy
    settings.enhanced_low_frequency_accuracy = not og_enhanced_low_frequency_accuracy
    new_enhanced_low_frequency_accuracy = settings.enhanced_low_frequency_accuracy

    print("done")


def do_hfss_sim_settings_test(settings: HFSSSimulationSettings):
    og_enabled = settings.enabled
    settings.enabled = not og_enabled
    new_enabled = settings.enabled

    do_general_test(settings.general)

    do_advanced_test(settings.advanced)

    do_advanced_meshing_test(settings.advanced_meshing)

    do_solver_test(settings.solver)

    do_dcr_test(settings.dcr)

    do_options_test(settings.options)


def do_test():
    test_db_path = "test.aedb"
    Database.delete(test_db_path)
    db = Database.create(test_db_path)
    test_cell = Cell.create(db, CellType.CIRCUIT_CELL, "test_cell")

    hfss_sim_setup = HfssSimulationSetup.create(test_cell, "test_sim_setup")

    do_hfss_sim_setup_test(hfss_sim_setup)

    do_hfss_sim_settings_test(hfss_sim_setup.settings)

    print("done")


def test_sim_setup():
    with session(settings.server_exe_dir(), 50051):
        do_test()



import settings

from ansys.edb.core.database import Database
from ansys.edb.core.definition.component_def import ComponentDef
from ansys.edb.core.definition.die_property import DieOrientation, DieProperty, DieType
from ansys.edb.core.definition.ic_component_property import ICComponentProperty
from ansys.edb.core.definition.io_component_property import IOComponentProperty
from ansys.edb.core.definition.package_def import PackageDef
from ansys.edb.core.definition.port_property import PortProperty
from ansys.edb.core.definition.rlc_component_property import RLCComponentProperty
from ansys.edb.core.definition.solder_ball_property import SolderBallProperty, SolderballShape
from ansys.edb.core.hierarchy.component_group import ComponentGroup, ComponentType
from ansys.edb.core.hierarchy.pin_pair_model import PinPairModel
from ansys.edb.core.layout.cell import Cell, CellType
from ansys.edb.core.session import session
from ansys.edb.core.utility.rlc import Rlc


def do_test():
    db = Database.create(settings.temp_dir())
    cell = Cell.create(db, CellType.CIRCUIT_CELL, "my_cell")
    layout = cell.layout

    print("Create Component group")
    fp = Cell.create(db, CellType.FOOTPRINT_CELL, "test-fp-cell")
    comp_def = ComponentDef.create(db, "test-comp-def", fp)
    comp_grp = ComponentGroup.create_with_component(layout, "test-cmp-grp", "test-comp-def")

    print("Create pin pair model")
    ppm = PinPairModel.create()
    rlc = Rlc(1, True)
    ppm.set_rlc(("test-pin-1", "test-pin-2"), rlc)

    print("Create rlc component property")
    rlc_cp = RLCComponentProperty.create()
    rlc_cp.enabled = False
    rlc_cp.package_mounting_offset = 0.5
    rlc_cp.package_def = PackageDef.create(db, "test-package-def")
    rlc_cp.model = ppm

    print("Change component group property")
    old_type = comp_grp.component_type
    comp_grp.component_type = ComponentType.RESISTOR
    comp_grp.component_property = rlc_cp

    print("------------RLC------------------")
    print("Previous component type was: " + str(old_type))
    print("New component type is: " + str(comp_grp.component_type))
    print("Is component property enabled: " + str(comp_grp.component_property.enabled))
    print("Package mounting offset is: " + str(comp_grp.component_property.package_mounting_offset))
    print("Package def is: " + str(comp_grp.component_property.package_def.name))
    print("---------------------------------\n")

    print("Create Solder ball property")
    sold_prop = SolderBallProperty.create()
    sold_prop.shape = SolderballShape.SOLDERBALL_CYLINDER
    sold_prop.height = 0.2
    sold_prop.set_diameter(0.1, 0.2)

    print("Create Die property")
    die_prop = DieProperty.create()
    die_prop.die_type = DieType.FLIPCHIP
    die_prop.die_orientation = DieOrientation.CHIP_DOWN

    print("Create Port property")
    port_prop = PortProperty.create()
    port_prop.reference_size_auto = True

    print("Create ic component property")
    ic_cp = ICComponentProperty.create()
    ic_cp.port_property = port_prop
    ic_cp.die_property = die_prop
    ic_cp.solder_ball_property = sold_prop

    print("Change component group property")
    old_type = comp_grp.component_type
    comp_grp.component_type = ComponentType.IC
    comp_grp.component_property = ic_cp

    print("-------------IC------------------")
    print("Previous component type was: " + str(old_type))
    print("New component type is: " + str(comp_grp.component_type))
    print("Package mounting offset is: " + str(comp_grp.component_property.package_mounting_offset))
    print(
        "Solder Property shape is: " + str(comp_grp.component_property.solder_ball_property.shape)
    )
    print(
        "Port Property height is: "
        + str(comp_grp.component_property.port_property.reference_height)
    )
    print(
        "Die Property orientation is: "
        + str(comp_grp.component_property.die_property.die_orientation)
    )
    print("---------------------------------\n")

    print("Create io component property")
    io_cp = IOComponentProperty.create()
    new_port_prop = port_prop.clone()
    new_port_prop.reference_height = 0.3
    new_solder_prop = sold_prop.clone()
    new_solder_prop.shape = SolderballShape.SOLDERBALL_SPHEROID
    io_cp.port_property = new_port_prop
    io_cp.solder_ball_property = new_solder_prop
    rlc_cp.package_mounting_offset = 0.7

    print("Change component group property")
    old_type = comp_grp.component_type
    comp_grp.component_type = ComponentType.IO
    comp_grp.component_property = io_cp

    print("-------------IO------------------")
    print("Previous component type was: " + str(old_type))
    print("New component type is: " + str(comp_grp.component_type))
    print("Package mounting offset is: " + str(comp_grp.component_property.package_mounting_offset))
    print(
        "Solder Property shape is: " + str(comp_grp.component_property.solder_ball_property.shape)
    )
    print(
        "Port Property height is: "
        + str(comp_grp.component_property.port_property.reference_height)
    )
    print("---------------------------------\n")

    comp_grp.delete()


def test_component_property():
    with session(settings.server_exe_dir(), 50051):
        do_test()



import settings

from ansys.edb.core.database import Database
from ansys.edb.core.layer.layer import Layer, LayerType
from ansys.edb.core.layer.layer_collection import LayerCollection, LayerCollectionMode
from ansys.edb.core.layout.cell import Cell, CellType
from ansys.edb.core.primitive.board_bend_def import (
    BoardBendDef,
    Rectangle,
    RectangleRepresentationType,
)
from ansys.edb.core.session import session
from ansys.edb.core.utility.value import Value


def do_test():
    db = Database.create("test.aedb")
    lyt = Cell.create(db, CellType.CIRCUIT_CELL, "test_cell").layout

    lc_col = LayerCollection.create(LayerCollectionMode.MULTIZONE)
    lc_col.add_layers([Layer.create("Outline", LayerType.OUTLINE_LAYER)])
    zone_id = lc_col.insert_zone(0)
    lyt.layer_collection = lc_col

    fixed_rect_props = (-1e-3, -1.0e-3, 0.0, 1e-3, 0.0, 0.0)
    rect_props = (0.0, -1.0e-3, 1e-2, 1e-3, 0.0, 0.0)
    #                 x1   ,  y1,  x2/w, y2/h, rad, rot

    fixed_zone_prim = Rectangle.create(
        lyt, "Outline", None, RectangleRepresentationType.LOWER_LEFT_UPPER_RIGHT, *fixed_rect_props
    )
    non_fixed_zone_prim = Rectangle.create(
        lyt, "Outline", None, RectangleRepresentationType.LOWER_LEFT_UPPER_RIGHT, *rect_props
    )

    lyt.synchronize_bend_manager()

    fixed_zone_prim.make_zone_primitive(0)
    non_fixed_zone_prim.make_zone_primitive(0)

    board_bend_def_props = ((5.0e-3, -1.0e-3), (5.0e-3, 1.0e-3)), Value("1mm"), Value("90deg")

    board_bend_def = BoardBendDef.create(lyt, non_fixed_zone_prim, *board_bend_def_props)

    lyt.synchronize_bend_manager()

    angle = board_bend_def.angle
    board_bend_def.angle = "95deg"
    new_angle = board_bend_def.angle

    radius = board_bend_def.radius
    board_bend_def.radius = "2mm"
    new_radius = board_bend_def.radius

    boundary_prim = board_bend_def.boundary_primitive

    bend_middle = board_bend_def.bend_middle
    board_bend_def.bend_middle = ((5.5e-3, -0.5e-3), (5.5e-3, 0.5e-3))
    new_bend_middle = board_bend_def.bend_middle

    bent_regions = board_bend_def.bent_regions

    print("done")


def test_board_bend_def():
    with session(settings.server_exe_dir(), 50051):
        do_test()



import random
import string

import pytest

from ansys.edb.core.layer.layer import Layer
from ansys.edb.core.layout.layout import Layout
from ansys.edb.core.net.net import Net

from .test_utils import create_edb_obj_msg, generate_random_int


@pytest.fixture
def mocked_stub(mocker):
    def _stub(mod, cls):
        mock = mocker.Mock()
        path = f"{mod.__name__}.{cls.__name__}._{cls.__name__}__stub"
        mocker.patch(path, mock)
        return mock

    return _stub


@pytest.fixture(params=[True, False])
def bool_val(request):
    """Parameterized fixture that returns True and False values

    Returns
    -------
    bool
    """
    return request.param


@pytest.fixture
def edb_obj_msg():
    """Fixture for creating an EDBObjMessage.

    Returns
    -------
    EDBObjMessage
    """
    return create_edb_obj_msg()


@pytest.fixture
def random_str():
    """Fixture for creating a 10 character long string comprised of random ascii letters.

    Returns
    -------
    str
    """
    return "".join(random.choice(string.ascii_letters) for _ in range(10))


@pytest.fixture
def random_int():
    """Fixture for generating a random integer between 0 and 100,000.

    Returns
    -------
    int
    """
    return generate_random_int()


@pytest.fixture
def layouts(request):
    count = request.param if hasattr(request, "param") and request.param else 1
    return [Layout(create_edb_obj_msg()) for _ in range(count)]


@pytest.fixture
def layout(layouts):
    return layouts[0]


@pytest.fixture
def nets(request):
    count = request.param if hasattr(request, "param") and request.param else 1
    return [Net(create_edb_obj_msg()) for _ in range(count)]


@pytest.fixture
def net(nets):
    return nets[0]


@pytest.fixture
def layers(request):
    count = request.param if hasattr(request, "param") and request.param else 1
    return [Layer(create_edb_obj_msg()) for _ in range(count)]


@pytest.fixture
def layer(layers):
    return layers[0]


@pytest.fixture
def layer_ref(request):
    if isinstance(request.param, str):
        ret = request.getfixturevalue(request.param)
        if isinstance(ret, Layer) or isinstance(ret, str):
            return ret
        else:
            comment = (
                f"Invalid parameter for 'layer_ref'."
                f"Fixture '{request.param}' resolved to {type(ret)}. Must be either str or Layer."
            )
    else:
        comment = (
            "Invalid parameter for 'layer_ref'."
            "It must be a string name of another fixture that resolves to str or Layer."
        )

    raise RuntimeError(comment)



"""Layout instance."""
from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ansys.edb.core.typing import LayerLike, NetLike

import ansys.api.edb.v1.layout_instance_pb2 as layout_instance_pb2

from ansys.edb.core.geometry.point_data import PointData
from ansys.edb.core.geometry.polygon_data import PolygonData
from ansys.edb.core.inner import ObjBase, utils
from ansys.edb.core.inner.messages import (
    layer_ref_message,
    net_ref_message,
    point_message,
    polygon_data_message,
    strings_message,
)
from ansys.edb.core.inner.utils import client_stream_iterator
from ansys.edb.core.layout_instance import layout_obj_instance
from ansys.edb.core.session import LayoutInstanceServiceStub, StubAccessor, StubType


class LayoutInstance(ObjBase):
    """Represents a layout instance object."""

    __stub: LayoutInstanceServiceStub = StubAccessor(StubType.layout_instance)

    def refresh(self):
        """Refresh the layout instance so it contains an up-to-date geometry."""
        self.__stub.Refresh(self.msg)

    @staticmethod
    def _query_request_iterator(requests):
        chunk_entry_creator = lambda request: request
        chunk_entries_getter = lambda chunk: chunk.queries
        return client_stream_iterator(
            requests,
            layout_instance_pb2.LayoutObjInstancesQueriesMessage,
            chunk_entry_creator,
            chunk_entries_getter,
        )

    def query_layout_obj_instances(
        self,
        layer_filter: LayerLike | list[LayerLike] = None,
        net_filter: NetLike | list[NetLike] = None,
        spatial_filter: PolygonData | PointData | None | list[PolygonData | PointData] = None,
    ):
        """Query layout object instances using the provided filters.

        Parameters
        ----------
        layer_filter : :term:`LayerLike` or list of :term:`LayerLike`, default: None
            Layers to query. The default is ``None``, in which case all layers are queried.
        net_filter : :term:`NetLike` or list of :term:`NetLike`, default: None
            Nets to query. The default is ``None``, in which case all nets are queried.
        spatial_filter : .PolygonData or .PointData or :obj:`None` or \
        list of .PolygonData or .PointData, default: None
            Area of the design to query. The default is :obj:`None`, in which case the entire
            spatial domain of the design is queried.

        Returns
        -------
        :term:`LayoutInstanceQueryResult` or list of :term:`LayoutInstanceQueryResult`
            If a single query is provided, one :term:`LayoutInstanceQueryResult` is returned. If a \
            list of spatial queries is provided, then a list of :term:`LayoutInstanceQueryResult` is \
            returned where each entry maps to the spatial query at the same index.
        """

        def to_msg_filter_list(client_filter, ref_msg_type):
            if client_filter is None:
                return None
            return utils.map_list(utils.ensure_is_list(client_filter), ref_msg_type)

        def get_spatial_filter_msg_params(_spatial_filter):
            if isinstance(_spatial_filter, PointData):
                return "point_filter", point_message(_spatial_filter)
            elif isinstance(_spatial_filter, PolygonData):
                return "region_filter", polygon_data_message(_spatial_filter)
            else:
                raise ValueError(
                    f"""Spatial filter of type {_spatial_filter.__class__.__name__} is not supported. """
                    """Only PointData and PolygonData are supported."""
                )

        def spatial_filter_to_msg(_spatial_filter):
            spatial_filter_msg_params = get_spatial_filter_msg_params(_spatial_filter)
            return layout_instance_pb2.LayoutObjInstancesQueryMessage(
                **{spatial_filter_msg_params[0]: spatial_filter_msg_params[1]}
            )

        # Create queries
        lyt_inst_net_filter_lyr_filter_params = {
            "layout_inst": self.msg,
            "layer_filter": to_msg_filter_list(layer_filter, layer_ref_message),
            "net_filter": to_msg_filter_list(net_filter, net_ref_message),
        }
        has_single_spatial_filter = spatial_filter is not None and not isinstance(
            spatial_filter, list
        )
        if has_single_spatial_filter:
            spatial_filter_msg_params = get_spatial_filter_msg_params(spatial_filter)
            lyt_inst_net_filter_lyr_filter_params[
                spatial_filter_msg_params[0]
            ] = spatial_filter_msg_params[1]
        requests = [
            layout_instance_pb2.LayoutObjInstancesQueryMessage(
                **lyt_inst_net_filter_lyr_filter_params
            )
        ]
        has_spatial_filter = spatial_filter is not None
        if has_spatial_filter and not has_single_spatial_filter:
            for sf in utils.ensure_is_list(spatial_filter):
                requests.append(spatial_filter_to_msg(sf))

        all_hits = []

        if len(requests) == 1:
            for hit in self.__stub.QueryLayoutObjInstances(requests[0]).query_results:
                all_hits.append(hit)
        else:
            for hits_chunk in self.__stub.StreamLayoutObjInstancesQuery(
                self._query_request_iterator(requests)
            ):
                for hit in hits_chunk.query_results:
                    all_hits.append(hit)

        def process_hits(_spatial_filter, hits_iter):
            full_hits = []
            partial_hits = []
            for hit in hits_iter:
                if hit.is_end_of_query_results_group:
                    break
                lyt_obj_inst = layout_obj_instance.LayoutObjInstance(hit.edb_obj)
                if hit.is_partial:
                    partial_hits.append(lyt_obj_inst)
                else:
                    full_hits.append(lyt_obj_inst)
            return (
                (full_hits, partial_hits) if isinstance(_spatial_filter, PolygonData) else full_hits
            )

        all_hits_iter = iter(all_hits)
        if not has_spatial_filter:
            return process_hits(None, all_hits_iter)
        elif has_single_spatial_filter:
            return process_hits(spatial_filter, all_hits_iter)
        elif len(spatial_filter) == 1:
            return process_hits(spatial_filter[0], all_hits_iter)
        else:
            return [process_hits(sf, all_hits_iter) for sf in spatial_filter]

    def get_layout_obj_instance_in_context(self, layout_obj, context):
        """Get the layout object instance of the given :term:`connectable <Connectable>` in the provided context.

        Parameters
        ----------
        layout_obj : :term:`Connectable <Connectable>`
            Layout object with the instances to search.
        context : list[str]
            List of strings specifying the :class:`context <.LayoutInstanceContext>` that the /
            layout object instance is retrieved from.

            .. seealso:: :func:`context <ansys.edb.core.layout_instance.layout_obj_instance.LayoutObjInstance.context>`

        Returns
        -------
        :class:`.LayoutObjInstance`
        """
        return layout_obj_instance.LayoutObjInstance(
            self.__stub.GetLayoutObjInstanceInContext(
                layout_instance_pb2.GetLayoutObjInstanceInContextMessage(
                    layout_inst=self.msg,
                    layout_obj=layout_obj.msg,
                    context=strings_message(context),
                )
            )
        )

    def get_connected_objects(self, origin_layout_obj_inst, touching_only):
        """Get the layout object instance connected to the origin layout object instance.

        Parameters
        ----------
        origin_layout_obj_inst : :class:`.LayoutObjInstance`
            Layout object instance that is to act as the origin to get connected objects from.
        touching_only : bool
            Whether to get only layout object instances touching the orgigin layout object instance
            on the placement layer of the origin layout object. If ``False`` all layout object
            instances across all layers that are electrically connected to the origin layout object
            instance are returned.

        Returns
        -------
        list[:class:`.LayoutObjInstance`]
           List of layout object instances connected to the origin layout object instance.
        """
        hits = self.__stub.GetConnectedObjects(
            layout_instance_pb2.GetConnectedObjectsMessage(
                layout_inst=self.msg,
                layout_obj=origin_layout_obj_inst.msg,
                touching_only=touching_only,
            )
        )
        return utils.map_list(hits.items, layout_obj_instance.LayoutObjInstance)



"""Dielectric material definition."""
from __future__ import annotations

from typing import List, Tuple

from ansys.api.edb.v1 import multipole_debye_model_pb2_grpc
import ansys.api.edb.v1.multipole_debye_model_pb2 as pb
from google.protobuf import empty_pb2

from ansys.edb.core import session
from ansys.edb.core.definition.dielectric_material_model import DielectricMaterialModel
from ansys.edb.core.inner import messages


class MultipoleDebyeModel(DielectricMaterialModel):
    """Represents a multipole Debye dielectric material model."""

    __stub: multipole_debye_model_pb2_grpc.MultipoleDebyeModelServiceStub = session.StubAccessor(
        session.StubType.multipole_debye_model
    )

    @classmethod
    def create(cls) -> MultipoleDebyeModel:
        """Create a multipole Debye dielectric material model.

        Returns
        -------
        .MultipoleDebyeModel
        """
        return MultipoleDebyeModel(cls.__stub.Create(empty_pb2.Empty()))

    @property
    def parameters(self) -> Tuple[List[float], List[float], List[float]]:
        """:obj:`tuple` of (:obj:`list` of :obj:`float`, :obj:`list` of :obj:`float`, :obj:`list` of :obj:`float`): \
        Parameters used to define the model.

        The tuple is of the form (``frequencies``, ``relative_permittivities``, ``dielectric_loss_tangents``) \
        where each relative permittivity and dielectric loss tangent value map to the frequency at the \
        same index as itself.
        """
        parameters_msg = self.__stub.GetParameters(messages.edb_obj_message(self))
        return (
            [i.value for i in parameters_msg.frequencies],
            [i.value for i in parameters_msg.relative_permitivities],
            [i.value for i in parameters_msg.loss_tangents],
        )

    @parameters.setter
    def parameters(self, params: Tuple[List[float], List[float], List[float]]):
        frequencies_msg = [messages.double_message(i) for i in params[0]]
        permitivities_msg = [messages.double_message(i) for i in params[1]]
        loss_tangents_msg = [messages.double_message(i) for i in params[2]]
        self.__stub.SetParameters(
            pb.MultipoleDebyeModelSetParams(
                target=self.msg,
                vectors=pb.MultipoleDebyeModelGetParams(
                    frequencies=frequencies_msg,
                    relative_permitivities=permitivities_msg,
                    loss_tangents=loss_tangents_msg,
                ),
            )
        )



import pytest
import settings

from ansys.edb.core.database import Database
from ansys.edb.core.definition.padstack_def import PadstackDef
from ansys.edb.core.definition.padstack_def_data import PadGeometryType, PadstackDefData, PadType
from ansys.edb.core.geometry.polygon_data import PolygonData
from ansys.edb.core.layer.layer import LayerType
from ansys.edb.core.layer.layer_collection import LayerCollectionMode
from ansys.edb.core.layer.stackup_layer import StackupLayer
from ansys.edb.core.layer.via_layer import ViaLayer
from ansys.edb.core.layout.cell import Cell, CellType
from ansys.edb.core.net.net import Net
from ansys.edb.core.primitive.path import Path as Line
from ansys.edb.core.primitive.path import PathCornerType, PathEndCapType
from ansys.edb.core.primitive.rectangle import Rectangle, RectangleRepresentationType
from ansys.edb.core.session import session
from ansys.edb.core.terminal.edge_terminal import EdgeTerminal, PrimitiveEdge


@pytest.fixture
def test_session():
    with session(settings.server_exe_dir()):
        yield


@pytest.fixture
def new_database_path(tmp_path):
    return str(tmp_path / "test.aedb")


@pytest.fixture
def new_database(test_session, new_database_path):
    db = Database.create(new_database_path)
    yield db
    db.close()


@pytest.fixture
def circuit_cell(new_database):
    return Cell.create(new_database, CellType.CIRCUIT_CELL, "circuit_cell")


@pytest.fixture()
def circuit_cell_with_overlapping_stackup(circuit_cell: Cell):
    layers = [
        StackupLayer.create("D1", LayerType.DIELECTRIC_LAYER, 150e-6, 0.0, "FR4_epoxy"),
        StackupLayer.create("L1", LayerType.CONDUCTING_LAYER, 25e-6, 125e-6, "copper"),
        StackupLayer.create("L2", LayerType.CONDUCTING_LAYER, 25e-6, 0.0, "copper"),
        ViaLayer.create("V1", "L2", "L1", "copper"),
    ]
    circuit_cell.layout.layer_collection.mode = LayerCollectionMode.OVERLAPPING
    for layer in layers:
        circuit_cell.layout.layer_collection.add_stackup_layer_at_elevation(layer)
    yield circuit_cell


@pytest.fixture()
def circuit_cell_with_stackup(circuit_cell: Cell):
    layers = [
        StackupLayer.create("L1", LayerType.CONDUCTING_LAYER, 25e-6, 0.0, "copper"),
        StackupLayer.create("D1", LayerType.DIELECTRIC_LAYER, 100e-6, 0.0, "FR4_epoxy"),
        StackupLayer.create("L2", LayerType.CONDUCTING_LAYER, 25e-6, 0.0, "copper"),
    ]
    for layer in layers:
        circuit_cell.layout.layer_collection.add_layer_bottom(layer)
    yield circuit_cell


@pytest.fixture()
def circuit_cell_with_padstack_def(circuit_cell_with_stackup: Cell):
    db = circuit_cell_with_stackup.database
    padstack_def_data = PadstackDefData.create()
    layer_names = ["Start", "Default", "Stop"]
    padstack_def_data.add_layers(layer_names)
    for name in layer_names:
        padstack_def_data.set_pad_parameters(
            name,
            PadType.REGULAR_PAD,
            0.0,
            0.0,
            0.0,
            PadGeometryType.PADGEOMTYPE_CIRCLE,
            [70e-6],
        )
        padstack_def_data.set_pad_parameters(
            name,
            PadType.ANTI_PAD,
            0.0,
            0.0,
            0.0,
            PadGeometryType.PADGEOMTYPE_CIRCLE,
            [100e-6],
        )
    padstack_def_data.set_hole_parameters(
        0.0, 0.0, 0.0, PadGeometryType.PADGEOMTYPE_CIRCLE, [50e-6]
    )
    padstack_def = PadstackDef.create(db, "VIA050070100")
    padstack_def.data = padstack_def_data
    yield circuit_cell_with_stackup


@pytest.fixture
def circuit_cell_with_edge_terminals(circuit_cell_with_padstack_def: Cell) -> Cell:
    _ = Rectangle.create(
        circuit_cell_with_padstack_def.layout,
        "L2",
        Net.create(circuit_cell_with_padstack_def.layout, "GND"),
        RectangleRepresentationType.CENTER_WIDTH_HEIGHT,
        0.0,
        0.0,
        2e-3,
        1e-3,
        0.0,
        0.0,
    )

    line = Line.create(
        circuit_cell_with_padstack_def.layout,
        "L1",
        Net.create(circuit_cell_with_padstack_def.layout, "NET1"),
        0.2e-3,
        PathEndCapType.FLAT,
        PathEndCapType.FLAT,
        PathCornerType.ROUND,
        PolygonData([[-0.9e-3, 0.0], [0.9e-3, 0.0]]),
    )

    edge1 = PrimitiveEdge.create(line, [-0.9e-3, 0.0])
    edge_terminal1 = EdgeTerminal.create(
        circuit_cell_with_padstack_def.layout,
        "P1",
        [edge1],
        Net.find_by_name(circuit_cell_with_padstack_def.layout, "NET1"),
    )
    edge_terminal1.reference_layer = "L2"

    edge2 = PrimitiveEdge.create(line, [0.9e-3, 0.0])
    edge_terminal2 = EdgeTerminal.create(
        circuit_cell_with_padstack_def.layout,
        "P2",
        [edge2],
        Net.find_by_name(circuit_cell_with_padstack_def.layout, "NET1"),
    )
    edge_terminal2.reference_layer = "L2"

    return circuit_cell_with_padstack_def



import ansys.api.edb.v1.layer_pb2 as layer_pb2
import ansys.api.edb.v1.term_pb2 as term_pb2
from utils.fixtures import *  # noqa
from utils.test_utils import create_edb_obj_collection_msg, equals

from ansys.edb.core.geometry.point_data import PointData
from ansys.edb.core.inner import messages
from ansys.edb.core.layer import layer as layer_api
import ansys.edb.core.terminal.bundle_terminal as bundle_terminal_mod
import ansys.edb.core.terminal.point_terminal as point_terminal_mod
import ansys.edb.core.terminal.terminal as terminal_mod
from ansys.edb.core.utility.port_post_processing_prop import PortPostProcessingProp
from ansys.edb.core.utility.rlc import Rlc


@pytest.fixture
def patch(mocker):
    def _patch(cls):
        mock = mocker.Mock()
        path = f"ansys.edb.core.terminal.terminals.{cls.__name__}._{cls.__name__}__stub"
        mocker.patch(path, mock)
        return mock

    return _patch


@pytest.fixture
def bundle_terminal(edb_obj_msg):
    return bundle_terminal_mod.BundleTerminal(edb_obj_msg)


@pytest.fixture
def point_terminal(edb_obj_msg):
    return point_terminal_mod.PointTerminal(edb_obj_msg)


@pytest.fixture
def lyr_type_mock(mocked_stub):
    lyr_mock = mocked_stub(layer_api, layer_api.Layer)
    lyr_mock.GetLayerType.return_value = layer_pb2.LayerTypeMessage(
        type=layer_api.LayerType.USER_LAYER.value
    )
    return lyr_mock


def test_bundle_terminal_create(mocked_stub, point_terminal, bundle_terminal, edb_obj_msg):
    mock = mocked_stub(bundle_terminal_mod, bundle_terminal_mod.BundleTerminal).Create
    mock.return_value = edb_obj_msg

    bt = bundle_terminal_mod.BundleTerminal.create([point_terminal, bundle_terminal])

    mock.assert_called_once_with(
        messages.edb_obj_collection_message([point_terminal, bundle_terminal])
    )
    assert isinstance(bt, bundle_terminal_mod.BundleTerminal)
    assert not bt.is_null
    assert bt.id == edb_obj_msg.id


@pytest.mark.parametrize(
    "term_type, term_cls",
    [
        (term_pb2.TerminalType.POINT_TERM, point_terminal_mod.PointTerminal),
        (term_pb2.TerminalType.BUNDLE_TERM, bundle_terminal_mod.BundleTerminal),
    ],
)
def test_bundle_terminal_get_terminals(mocked_stub, bundle_terminal, term_type, term_cls):
    get_terminals = mocked_stub(
        bundle_terminal_mod, bundle_terminal_mod.BundleTerminal
    ).GetTerminals
    get_terminals.return_value = expected = create_edb_obj_collection_msg(2)
    get_params = mocked_stub(terminal_mod, terminal_mod.Terminal).GetParams
    get_params.return_value = term_pb2.TermParamsMessage(term_type=term_type)

    terms = bundle_terminal.terminals
    get_terminals.assert_called_once_with(bundle_terminal.msg)
    get_params.assert_called()

    assert len(terms) == 2
    for t in terms:
        assert isinstance(t, term_cls)
    assert sorted([t.id for t in terms]) == sorted([msg.id for msg in expected.items])


def test_bundle_terminal_ungroup(mocked_stub, bundle_terminal):
    expected = bundle_terminal.msg
    mock = mocked_stub(bundle_terminal_mod, bundle_terminal_mod.BundleTerminal).Ungroup
    mock.return_value = None

    bundle_terminal.ungroup()
    mock.assert_called_once_with(expected)
    assert bundle_terminal.is_null


def test_point_terminal_create(mocked_stub, layout, net, layer, edb_obj_msg):
    mock = mocked_stub(point_terminal_mod, point_terminal_mod.PointTerminal).Create
    mock.return_value = edb_obj_msg

    pt = point_terminal_mod.PointTerminal.create(layout, net, layer, "test-point-term", (1, 2))

    mock.assert_called_once_with(
        messages.point_term_creation_message(layout, net, layer, "test-point-term", (1, 2))
    )
    assert isinstance(pt, point_terminal_mod.PointTerminal)
    assert not pt.is_null
    assert pt.id == edb_obj_msg.id


def test_point_terminal_get_params(mocked_stub, point_terminal, layer, lyr_type_mock):
    point = (1e-9, 2e-9)
    mock = mocked_stub(point_terminal_mod, point_terminal_mod.PointTerminal).GetParameters
    mock.return_value = messages.point_term_params_message(layer, point)

    res = point_terminal.params

    mock.assert_called_once_with(point_terminal.msg)
    assert isinstance(res, tuple)
    assert len(res) == 2
    assert isinstance(res[0], Layer)
    assert isinstance(res[1], PointData)
    assert res[0].id == layer.id
    assert res[1].x == point[0]
    assert res[1].y == point[1]


@pytest.mark.parametrize(
    "layer_ref, point, success",
    [
        ("layer", (1e-9, 2e-9), True),  # Layer instance fixture & float & success
        ("random_str", (5, 6), False),  # Layer str name fixture & int   & fail
    ],
    indirect=["layer_ref"],
)
def test_point_terminal_set_params(mocked_stub, point_terminal, layer_ref, point, success):
    mock = mocked_stub(point_terminal_mod, point_terminal_mod.PointTerminal).SetParameters
    mock.return_value = messages.bool_message(success)

    point_terminal.params = (layer_ref, point)

    mock.assert_called_once_with(
        messages.point_term_set_params_message(point_terminal, layer_ref, point)
    )


def test_terminal_get_params(mocked_stub, point_terminal, bundle_terminal, layer, lyr_type_mock):
    mock = mocked_stub(terminal_mod, terminal_mod.Terminal)
    mock.GetParams.return_value = term_pb2.TermParamsMessage(
        term_type=term_pb2.TerminalType.POINT_TERM,
        boundary_type=terminal_mod.BoundaryType.RLC.value,
        term_to_ground=terminal_mod.SourceTermToGroundType.POSITIVE.value,
        hfss_pi_type=terminal_mod.HfssPIType.COAXIAL_SHORTENED.value,
        ref_layer=layer.msg,
        bundle_term=bundle_terminal.msg,
        is_interface=True,
        is_reference=False,
        is_auto_port=False,
        is_circuit_port=False,
        use_ref_from_hierarchy=True,
        name="name",
        impedance=messages.value_message(50),
        source_amplitude=messages.value_message(10),
        source_phase=messages.value_message(5),
        s_param_model="model",
        rlc=messages.rlc_message(Rlc(r=10, r_enabled=True)),
        port_post_processing_prop=messages.port_post_processing_prop_message(
            PortPostProcessingProp(renormalization_impedance=50, do_renormalize=True)
        ),
    )

    assert point_terminal.boundary_type == terminal_mod.BoundaryType.RLC
    assert point_terminal.term_to_ground == terminal_mod.SourceTermToGroundType.POSITIVE
    assert point_terminal.hfss_pi_type == terminal_mod.HfssPIType.COAXIAL_SHORTENED
    assert point_terminal.reference_terminal is None
    assert equals(point_terminal.reference_layer, layer)
    assert equals(point_terminal.bundle_terminal, bundle_terminal)
    assert point_terminal.is_interface_terminal
    assert not point_terminal.is_reference_terminal
    assert not point_terminal.is_auto_port
    assert not point_terminal.is_circuit_port
    assert point_terminal.use_reference_from_hierarchy
    assert point_terminal.name == "name"
    assert point_terminal.impedance.double == 50.0
    assert point_terminal.source_amplitude.double == 10.0
    assert point_terminal.source_phase.double == 5.0
    assert point_terminal.model == "model"
    rlc = point_terminal.rlc_boundary_parameters
    assert rlc is not None
    assert rlc.r.double == 10.0
    assert rlc.r_enabled and rlc.is_parallel
    assert not rlc.l_enabled and not rlc.c_enabled
    prop = point_terminal.port_post_processing_prop
    assert prop is not None
    assert prop.renormalization_impedance.double == 50.0
    assert prop.do_renormalize
    assert not prop.do_deembed and not prop.do_deembed_gap_l

    assert mock.GetParams.call_count == 18


def test_terminal_set_params(mocked_stub, point_terminal, bundle_terminal, layer):
    mock = mocked_stub(terminal_mod, terminal_mod.Terminal).SetParams
    mock.return_value = None

    point_terminal.boundary_type = terminal_mod.BoundaryType.RLC
    point_terminal.term_to_ground = terminal_mod.SourceTermToGroundType.NO_GROUND
    point_terminal.hfss_pi_type = terminal_mod.HfssPIType.COAXIAL_OPEN
    point_terminal.is_circuit_port = True
    point_terminal.is_auto_port = False
    point_terminal.use_reference_from_hierarchy = True
    point_terminal.name = "new-name"
    point_terminal.reference_terminal = None
    point_terminal.reference_terminal = bundle_terminal
    point_terminal.reference_layer = None
    point_terminal.reference_layer = layer
    point_terminal.reference_layer = layer, "context1"
    point_terminal.impedance = 50
    point_terminal.source_amplitude = 10
    point_terminal.source_phase = 5
    point_terminal.model = "new-model"
    point_terminal.rlc_boundary_parameters = Rlc(r=5, l=10, c=15, r_enabled=True, c_enabled=True)
    point_terminal.port_post_processing_prop = PortPostProcessingProp(
        voltage_magnitude=10, renormalization_impedance=50, do_renormalize=True
    )
    assert mock.call_count == 18
